VERSION 5.00
Begin VB.Form Form2 
   Caption         =   "Homemade Spreadsheet"
   ClientHeight    =   3690
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   6990
   LinkTopic       =   "Form2"
   ScaleHeight     =   3690
   ScaleWidth      =   6990
   StartUpPosition =   3  'Windows Default
   Begin VB.PictureBox picStatus 
      Align           =   2  'Align Bottom
      AutoRedraw      =   -1  'True
      Height          =   300
      Left            =   0
      ScaleHeight     =   240
      ScaleWidth      =   6930
      TabIndex        =   3
      Top             =   3390
      Width           =   6990
   End
   Begin VB.TextBox txtCell 
      Alignment       =   1  'Right Justify
      Height          =   285
      Index           =   0
      Left            =   360
      TabIndex        =   0
      Top             =   240
      Width           =   1575
   End
   Begin VB.Label lblRow 
      Alignment       =   2  'Center
      BorderStyle     =   1  'Fixed Single
      Height          =   255
      Index           =   0
      Left            =   0
      TabIndex        =   2
      Top             =   0
      Width           =   375
   End
   Begin VB.Label lblCol 
      Alignment       =   2  'Center
      BorderStyle     =   1  'Fixed Single
      Caption         =   "A"
      Height          =   255
      Index           =   1
      Left            =   360
      TabIndex        =   1
      Top             =   0
      Width           =   1575
   End
End
Attribute VB_Name = "Form2"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'-------------------------------------------------------
'  A VERY VERY SIMPLE SPREADSHEET
'-------------------------------------------------------

Option Explicit

Private Rows As Integer
Private Cols As Integer
Private vars() As New CVariable
Private expr() As New CExpression

Private activeCell As Integer
Private firstKey As Boolean

Private Sub Form_Load()
    CreateSpreadsheet 10, 4
End Sub

Sub StatusMessage(Optional text As String)
    picStatus.Cls
    picStatus.Print text
End Sub

Sub CreateSpreadsheet(numRows As Integer, numCols As Integer)
    Dim r As Integer, c As Integer
    Dim i As Integer, j As Integer
    
    Screen.MousePointer = vbHourglass
    
    ' this version only supports up to 10 columns and 30 rows
    Rows = numRows
    Cols = numCols
    If Cols > 10 Then Cols = 10
    If Rows > 30 Then Rows = 30
    
    ' prepare the matrix (actually an array) of expressions
    ReDim expr(0 To Rows * Cols - 1) As New CExpression
    ' ...and variables
    ReDim vars(0 To Rows * Cols - 1) As New CVariable
    
    ' creates the row of column headers
    For c = 2 To Cols
        Load lblCol(c)
        lblCol(c).Left = lblCol(c - 1).Left + lblCol(c - 1).Width
        lblCol(c).Caption = Chr$(64 + c)
        lblCol(c).Visible = True
    Next
    ' creates column of row headers
    For r = 1 To Rows
        Load lblRow(r)
        lblRow(r).Top = lblRow(r - 1).Top + lblRow(r - 1).Height
        lblRow(r).Caption = Format$(r)
        lblRow(r).Visible = True
    Next
    ' creates the grid of cells
    For r = 1 To Rows
        For c = 1 To Cols
            ' don't create the first textbox
            If i > 0 Then Load txtCell(i)
            txtCell(i).Move lblCol(c).Left, lblRow(r).Top, lblCol(c).Width, lblRow(c).Height
            txtCell(i).Visible = True
            ' create the corresponding CVariable object
            vars(i).Name = Chr$(c + 64) & Format$(r)
            i = i + 1
        Next
    Next
    
    StatusMessage "Initializing..."
    Show
    DoEvents
    
    ' link all variables to all expressions
    For i = 0 To Rows * Cols - 1
        For j = 0 To Rows * Cols - 1
            expr(i).AutoCreateVariables = False
            expr(i).AddVariable vars(j)
        Next
    Next
    
    Screen.MousePointer = vbDefault
    
    StatusMessage "(Use a leading '=' for entering formulas)"

End Sub

Private Sub txtCell_KeyPress(Index As Integer, KeyAscii As Integer)
    ' Enter key moves one cell down
    If KeyAscii = 13 Then
        MoveCell "D"
        KeyAscii = 0   ' avoids the beep
    End If
    firstKey = False
End Sub

Private Sub txtCell_KeyDown(Index As Integer, KeyCode As Integer, Shift As Integer)
    If KeyCode = vbKeyDown Then
        MoveCell "D"
    ElseIf KeyCode = vbKeyUp Then
        MoveCell "U"
    ElseIf KeyCode = vbKeyLeft Then
        If firstKey Then MoveCell "L"
    ElseIf KeyCode = vbKeyRight Then
        If firstKey Then MoveCell "R"
    End If
    firstKey = False
End Sub

Private Sub txtCell_GotFocus(Index As Integer)
    Dim r As Integer, c As Integer
    ' highlight the markers on row/col headers
    r = (Index \ Cols) + 1
    c = (Index Mod Cols) + 1
    txtCell(Index).BackColor = vbCyan
    lblRow(r).BackColor = vbCyan
    lblCol(c).BackColor = vbCyan
    
    activeCell = Index
    firstKey = True
    ' show the contents of the formula on the status bar
    StatusMessage expr(Index).Expression
End Sub


Private Sub txtCell_LostFocus(Index As Integer)
    Dim r As Integer, c As Integer
    Dim saveExpr As String
    
    ' unlight the markers on row/col headers
    r = (Index \ Cols) + 1
    c = (Index Mod Cols) + 1
    txtCell(Index).BackColor = vbWindowBackground
    lblRow(r).BackColor = vb3DFace
    lblCol(c).BackColor = vb3DFace
    
    'if the cell has been changed
    If txtCell(Index).DataChanged Then
        If Left$(txtCell(Index), 1) = "=" Then
            ' it is a formula
            saveExpr = expr(Index).Expression
            expr(Index).Expression = Mid$(txtCell(Index), 2)
            If expr(Index).ErrorCode Then
                expr(Index).Expression = saveExpr
                MsgBox "Invalid formula", vbExclamation
                txtCell(Index).SetFocus
            Else
                RecalcSpreadsheet
            End If
        Else
            ' assign the value of the cell to the corresponding variable
            If IsNumeric(txtCell(Index)) Then
                vars(Index).Value = Val(txtCell(Index))
            End If
            ' recalc the whole spreadsheet
            RecalcSpreadsheet
        End If
        ' clear the flag
        txtCell(Index).DataChanged = False
    End If
End Sub

Sub RecalcSpreadsheet()
    Dim Index As Integer
    
    For Index = 0 To Rows * Cols - 1
        If expr(Index).Expression <> "" Then
            vars(Index).Value = expr(Index).Value
            txtCell(Index).text = Format(vars(Index).Value)
        End If
    Next
End Sub

Sub MoveCell(direction As String)
    Dim r As Integer, c As Integer
    
    r = (activeCell \ Cols) + 1
    c = (activeCell Mod Cols) + 1
    Select Case direction
        Case "L"
            If c > 1 Then txtCell(activeCell - 1).SetFocus
        Case "R"
            If c < Cols Then txtCell(activeCell + 1).SetFocus
        Case "U"
            If r > 1 Then txtCell(activeCell - Cols).SetFocus
        Case "D"
            If r < Rows Then txtCell(activeCell + Cols).SetFocus
    End Select
    
End Sub
